import logging
import json

import redis

from .registrar import register_file


logger = logging.getLogger(__name__)


def run_daemon(config, replace, host, port, listen_queue, registered_set_key):
    """ Run the registrar daemon, listening on a redis queue
        for files to be registered. After preprocessing the filename
        of the registered files will be pushed to the output queue.
    """
    # initialize the queue client
    client = redis.Redis(
        host=host, port=port, charset="utf-8", decode_responses=True
    )
    logger.debug("waiting for redis queue '%s'..." % listen_queue)
    while True:
        # fetch an item from the queue to be registered
        _, value = client.brpop(listen_queue)
        # start the registration on that file
        try:
            item = register_file(config, value, replace)
            client.sadd(registered_set_key, item.identifier)

        except Exception as e:
            logger.exception(e)
