from os import remove
from os.path import join, basename
from tempfile import gettempdir, gettempprefix
from dataclasses import dataclass, field
from typing import Union, Type, Optional, List, Callable, Any
import logging

import lxml.etree

from .source import Source
from .exceptions import RegistrationError


logger = logging.getLogger(__name__)

def read_xml(source: Source, path: str) -> lxml.etree._ElementTree:
    out_filename = join(gettempdir(), basename(path))
    try:
        source.get_file(path, out_filename)
        tree = lxml.etree.parse(out_filename)
    finally:
        remove(out_filename)
    return tree

@dataclass
class Parameter:
    xpath: str
    multi: bool = False
    parser: Optional[Callable[[str], Any]] = None
    namespaces: dict = field(default_factory=dict)


class ParserError(RegistrationError):
    pass

def parse_metadata_schema(tree: lxml.etree._ElementTree, schema: dict, namespaces: dict=None) -> dict:
    out = {}
    for key, param in schema.items():
        values = tree.xpath(param.xpath, namespaces=param.namespaces or namespaces)
        if param.multi:
            value = [
                param.parser(v) if param.parser else v
                for v in values
            ]
        else:
            try:
                value = param.parser(values[0]) if param.parser else values[0]
            except IndexError:
                raise ParserError(f'Failed to fetch single value for parameter {key}')

        out[key] = value

    return out
